import java.applet.*;
import java.awt.*;
import java.net.URL;
import ach.ImagePanel;
import ach.BevelPanel;
import ach.file.ParamUtil;

/**
 * TiffyApplet.java                                                   <br>
 * Copyright (c) 1997-2006 Helge Hackbarth, All Rights Reserved.      <br>
 * Main class for an applet demonstrating the Tiffy Java Toolkit      <br>
 *                                                                    <br>
 * THE AUTHOR MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
 * OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. THE AUTHOR SHALL NOT BE LIABLE FOR ANY
 * DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 *
 * @author Helge Hackbarth
 * @version 3.1.3
 */
public class TiffyApplet extends Applet
{
  ImagePanel imagePanel; // panel containing image and control components

  // PARAMETER SUPPORT:
  // Parameters allow an HTML author to pass information to the applet;
  // the HTML author specifies them using the <PARAM> tag within the <APPLET>
  // tag.  The following variables are used to store the values of the
  // parameters.
  //--------------------------------------------------------------------------

  // Members for applet parameters
  // <type>       <MemberVar>    = <Default Value>
  //--------------------------------------------------------------------------
  private String m_Filename = "";
  private String m_Options = "";

  // Parameter names.  To change a name of a parameter, you need only make
  // a single change.  Simply modify the value of the parameter string below.
  //--------------------------------------------------------------------------
  private final String PARAM_Filename = "Filename";
  private final String PARAM_Options = "Options";

  // The GetParameters() method retrieves the values of each of the applet's
  // parameters and stores them in variables.
  // When the applet is run within an HTML page, this method is called by the
  // init() method
  //---------------------------------------------------------------------------
  void GetParameters()
  {
    // Query values of all Parameters
    //--------------------------------------------------------------
    String param;

    // Filename: Name of image file
    //--------------------------------------------------------------
    param = getParameter(PARAM_Filename);
    if (param != null)
      m_Filename = param;

    // Options: Options for display
    //--------------------------------------------------------------
    param = getParameter(PARAM_Options);
    if (param != null)
      m_Options = param;

  }


  // TiffyApplet Class Constructor
  //--------------------------------------------------------------------------
  public TiffyApplet()
  {
    // You may add constructor code here
  }

  // The init() method is called by the AWT when an applet is first loaded or
  // reloaded.
  //--------------------------------------------------------------------------
  public void init()
  {
    GetParameters();
    setBackground(Color.lightGray);
    setLayout(new BorderLayout());
    // To allow loading toolkit related ressources compatible to Java 1.0
    // use ParamUtil.retrieveCodeBase(this) for applets and
    // ParamUtil.retrieveProgramPath(this, defaultPath) for standalone applications
    // This is important for applets running inside Netscape Navigator as well
    ParamUtil.retrieveCodeBase(this);
    try  {
      if (m_Filename.indexOf(';') >= 0) // parameter is a ;-separated list of names
        imagePanel = new ImagePanel(this, m_Filename, m_Options, true);
      else {
        URL context = new URL(getDocumentBase(), m_Filename);
        imagePanel = new ImagePanel(this, context, m_Options, true);
      }
      BevelPanel bPanel = new BevelPanel(BevelPanel.UP);
      bPanel.add(imagePanel);
      add("Center",bPanel);
      // alternatively without 3D-Borders:
      //setLayout(new BorderLayout());
      //add("Center", imagePanel);
    } catch (java.net.MalformedURLException mue)
    {
      add(new Label("Can not access image URL \"" + m_Filename + "\""));
      // System.err.println(mue.toString());
    }
  }

  // Place additional applet clean up code here.  destroy() is called when
  // when your applet is terminating and being unloaded.
  //-------------------------------------------------------------------------
  public void destroy()
  {
    if (imagePanel != null)
      try { imagePanel.cleanUp(); } catch (Throwable t) { }
    // You may place applet cleanup code here
    if(System.getProperty("java.version").equals("1.1.5")) {
      // Dan Kok's workaround for Netscape 4.5-4.7/MSWINDOWS freezing bug
      try { Thread.sleep(2000); } catch(InterruptedException ex) { }
    }
  }

  // The start() method is called when the page containing the applet
  // first appears on the screen.
  //--------------------------------------------------------------------------
  public void start()
  {
    if (ach.OkBox.currentOkBox != null) ach.OkBox.showCurrentOkBox();
  }

  // The stop() method is called when the page containing the applet is
  // no longer on the screen.
  //--------------------------------------------------------------------------
  public void stop()
  {
    // As a modal dialog window opened from a browser applet does not
    // behave as expected, dispose it, if it is still open
    if (ach.OkBox.currentOkBox != null) ach.OkBox.currentOkBox.cleanUp();
    if (imagePanel != null)
      imagePanel.cleanUp();
    imagePanel = null;
    System.gc(); // force garbage collection
    // you may place additional applet stop code here
  }

  // A public method to zoom into an image demonstrating how
  // to use scripting from JavaScript to an applet
  public void zoomIn()
  {
    imagePanel.zoomIn();
  }

  // A public method to zoom out from an image demonstrating how
  // to use scripting from JavaScript to an applet
  public void zoomOut()
  {
    imagePanel.zoomOut();
  }

  // A public method to zoom into an image with a particular value
  // demonstrating how to use scripting from JavaScript to an applet
  public void zoomTo(double zoomFactor)
  {
    imagePanel.zoom(zoomFactor); ;
  }

  // A public method to load a new image (ImagePanel must already be instiated)
  // demonstrating how to use scripting from JavaScript to an applet
  public void loadImage(String name)
  {
    ach.image.ImageInfo imgInfo = new ach.image.ImageInfo();
    imgInfo.imgName = name;
    try { imgInfo.imgURL = new URL(getDocumentBase(), name); }
    catch (java.net.MalformedURLException mue) { System.err.println(mue.toString()); };
    imagePanel.loadImage(imgInfo);
  }

  // A public method to set annotations remotely e.g. through JavaScript.
  // Annotations are represented by an XML string in the format specified
  // in the documentation (TiffyAppletDoc.html).
  public void setAnnotations(String annotations)
  {
    imagePanel.setAnnotations(annotations);
  }

  // A public method to get currently loaded annotations remotely e.g. through JavaScript.
  // Annotations are returned as an XML string in the format specified
  // in the documentation (TiffyAppletDoc.html).
  public String getAnnotations(boolean resetModifiedState)
  {
    return imagePanel.getAnnotations(resetModifiedState);
  }

  /**
   * Toggle display of thumbnail preview
   */
  public void toggleThumbnails() {
    imagePanel.toggleThumbnails();
  }
}
